package pmicrovenus;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Observable;
import java.util.Vector;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.swing.JOptionPane;
  
@Entity
public class Jugador extends Observable implements Serializable{

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    
    private int idJugador;
    private String nombre;
    private float saldo;
    private int encarcelado;
    private int dobles;
    private int cogeCarta;
    
    @OneToOne(cascade=CascadeType.ALL)
    private Ficha ficha;
    
    @OneToOne(cascade=CascadeType.ALL)
    private Casilla posicion;
    
    @OneToMany(cascade=CascadeType.ALL)
    private List<Solar> solar;
    
    @OneToMany(cascade=CascadeType.ALL)
    private List<Servicio> servicio;
    
    @OneToMany(cascade=CascadeType.ALL)
    private List<Estacion> estacion;
    
    @OneToMany(cascade=CascadeType.ALL)
    private List<CartaSalidaCarcel> cartasalidacarcel;
   

    public Jugador() {
        encarcelado = 0;
        solar = new ArrayList<Solar>();
        servicio = new ArrayList<Servicio>();
        estacion = new ArrayList<Estacion>();
        dobles = 0;
        cartasalidacarcel=new ArrayList<CartaSalidaCarcel>();
    }
    public Jugador(int i,String nombre, float saldo,Ficha ficha) {
        this.idJugador = i;
        this.nombre = nombre;
        this.saldo = saldo;
        this.ficha = ficha;
        encarcelado = 0;
        solar = new ArrayList<Solar>();
        servicio = new ArrayList<Servicio>();
        estacion = new ArrayList<Estacion>();
        dobles = 0;
        cartasalidacarcel=new ArrayList<CartaSalidaCarcel>();
    }
    
    public String getNombre() {
        return nombre;
    }
    public void setNombre(String val) {
        this.nombre = val;
        setChanged();
        notifyObservers();
    }
    
    public float getSaldo() {
        return saldo;
    }
    public void setSaldo(float val) {
        this.saldo = val;
        setChanged();
        notifyObservers();
    }
    
    public boolean tieneEfectivo(float cantidad) {
        return (cantidad <= saldo);
    }
    
    public boolean isEncarcelado() {
        return encarcelado>0;
    }
    
    public int getEncarcelado() {
        return encarcelado;
    }
    
    public void setEncarcelado(int val) {
        this.encarcelado = val;
        setChanged();
        notifyObservers();
    }
    
    public boolean isEnBancarrota() {
        return saldo < 0;
    }
  
    public void ingreso(float val) {
        this.saldo =this.saldo + val;
        setChanged();
        notifyObservers();
    }
    public void reintegro(float val) {
        this.saldo =this.saldo - val;
        setChanged();
        notifyObservers();
    }
    
    public int getIdJugador() {
        return idJugador;
    }
    public void setIdJugador(int idJugador) {
        this.idJugador = idJugador;
        setChanged();
        notifyObservers();
    }
    
    public Casilla getPosicion() {
        return posicion;
    }
    public void setPosicion(Casilla pos) {
        this.posicion = pos;
        setChanged();
        notifyObservers();
    }
    
    public Ficha getFicha() {
        return ficha;
    }
    public void setFicha(Ficha ficha) {
        this.ficha = ficha;
        setChanged();
        notifyObservers();
    }
    
    public int getNumSolares() {
        return getSolares().size();
    }
        
    public int getNumEstaciones() {
        return estacion.size();
    }
      
    public int getNumServicios() {
        return servicio.size();
    }
    
    public List<Casilla> getTodasPropiedades() {
        ArrayList<Casilla> list = new ArrayList();
        list.addAll(solar);
        list.addAll(estacion);
        list.addAll(servicio);
        return (list);
    }
    public Vector<Casilla> getTodasPropiedadesV() {
        Vector<Casilla> list = new Vector();
        list.addAll(solar);
        list.addAll(estacion);
        list.addAll(servicio);
        return (list);
    }
     
    public Solar getSolar(String propiedad) {
        for(int i=0;i<getSolares().size();i++) {
            Solar s = getSolares().get(i);
            if(s.getDescripcion().compareTo(propiedad)==0) {
                    return s;
                }
        }
        return null;	
    }
    
     public Casilla getPropiedad(String propiedad) {
        List<Casilla> propiedades = new ArrayList();
        propiedades = getTodasPropiedades();
         for(int i=0;i<propiedades.size();i++) {
            Casilla c = propiedades.get(i);
            if(c.getDescripcion().compareTo(propiedad)==0) {
                    return c;
                }
        }
        return null;	
    }
    
    public boolean tieneSolar(String propiedad) {
        for(int i=0;i<getSolares().size();i++) {
            Casilla c = (Casilla)getSolares().get(i);
            if(c.getDescripcion().equals(propiedad)) {
                    return true;
                }
        }
        return false;	
	}
      
    public boolean tieneEstacion(String propiedad) {
        for(int i=0;i<estacion.size();i++) {
            Casilla c = (Casilla)estacion.get(i);
            if(c.getDescripcion().equals(propiedad)) {
                    return true;
                }
        }
        return false;	
	}
     
    public boolean tieneServicio(String propiedad) {
        for(int i=0;i<servicio.size();i++) {
            Casilla c = (Casilla)servicio.get(i);
            if(c.getDescripcion().equals(propiedad)) {
                    return true;
                }
        }
        return false;	
	}
   
    public void pagarRenta(){
        float renta=posicion.getRenta();
        Jugador j = posicion.getPropietario();
        
        if(saldo < renta) {
		j.ingreso(saldo);//propietario.saldo += saldo;
		reintegro(renta);
	}
	else {
		reintegro(renta);
                j.ingreso(renta);
	}
//	if(isEnBancarrota()) {
////		saldo = -1;
//		cambiarPropietario(j);
//	}
        
        System.out.println("Has pagado "+posicion.getRenta()+" al jugador "+ posicion.getPropietario().getNombre());                      
    }
    
    public void pagarRentaA(Jugador j,float renta){
        if(saldo < renta) {
		j.ingreso(saldo);
		reintegro(renta);
	}
	else {
                j.ingreso(renta);
		reintegro(renta);
	}
//      if(isEnBancarrota()) {
////		saldo = -1;
//		cambiarPropietario(j);
//	}
        
        String mens = "Has pagado "+ renta +" de renta a "+ (j.getNombre());
        String tit = getNombre();
        JOptionPane.showMessageDialog(null,mens,tit,JOptionPane.INFORMATION_MESSAGE);
        
        System.out.println("Has pagado "+ renta +" a "+ j.getNombre());                       
    }
     
    public void comprarPropiedad(Casilla propiedad, float precio){
        
        propiedad.setPropietario(this);
        if (propiedad instanceof Solar){
            getSolares().add((Solar) propiedad); 
        }
        if (propiedad instanceof Servicio){
            servicio.add((Servicio) propiedad);
        }
        if (propiedad instanceof Estacion){
            estacion.add((Estacion) propiedad);
        }
        reintegro(precio);
        System.out.println("Has comprado " + propiedad.getDescripcion()+ " por " + precio + " " );
        
        String mens = "Has comprado " + propiedad.getDescripcion()+ " por " + precio + " " ;
        String tit = getNombre();
        JOptionPane.showMessageDialog(null,mens,tit,JOptionPane.INFORMATION_MESSAGE);
        //System.out.println("Su nuevo saldo es " + saldo);
    }
    
    public void comprar() {
        Casilla c = getPosicion();
        if(c.getPropietario()==null) {
           if(this.tieneEfectivo(c.getPrecioCompra())){//c.getPrecioCompra() <= saldo) {
                if(c instanceof Solar) {
                    comprarSolar((Solar)c);
                }
                if(c instanceof Estacion) {
                    comprarEstacion((Estacion)c);
                }
                if(c instanceof Servicio) {
                    comprarServicio((Servicio)c);
                }
            }
        }
    }
     
    private void comprarSolar(Solar propiedad) {
        comprarPropiedad(propiedad, propiedad.getPrecioCompra());
    }
    private void comprarEstacion(Estacion propiedad) {
        comprarPropiedad(propiedad, propiedad.getPrecioCompra());
    }
    private void comprarServicio(Servicio propiedad) {
        comprarPropiedad(propiedad, propiedad.getPrecioCompra());
    }
  
    public void venderPropiedadA(Casilla propiedad, float dinero, Jugador comprador) {
        
        if(propiedad instanceof Solar) {
            getSolares().remove(propiedad);
        }
        if(propiedad instanceof Estacion) {
            estacion.remove(propiedad);
        }
        if(propiedad instanceof Servicio) {
            servicio.remove(propiedad);
        }
        comprador.comprarPropiedad(propiedad, dinero);
        ingreso(dinero);
    }
   
    public void venderConstruccion(Solar s) {
    
        if (!tieneSolar(s.getDescripcion())) return;
        
        if (s.getNCasas()==5)
        {
            s.setNCasas(s.getNCasas()-1);
            ingreso(s.getPrecioHotel()/2);
        } else
        if (s.getNCasas() > 0)
        {
            s.setNCasas(s.getNCasas()-1);
            ingreso(s.getPrecioCasa()/2);
        }
    }
    
    public List<Solar> getSolaresConstruidos() {
        
        ArrayList<Solar> solaresConstruidos = new ArrayList<Solar>();           
        Solar s = null;
                
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < solar.size(); i++)
        {
            s = solar.get(i);
            // obtenemos el n maximo y minimo de casas
            int maxNumCasas = 0;
            int minNumCasas = 0;

            if (s.getNCasas() > maxNumCasas)
                    maxNumCasas = s.getNCasas();
            for (int j = 0; j < s.getAsociados().size(); j++)
            {
                if (s.getAsociados().get(j).getNCasas() > maxNumCasas)
                    maxNumCasas = s.getAsociados().get(j).getNCasas();
            }
            minNumCasas = maxNumCasas;
            if (s.getNCasas() < minNumCasas)
                    minNumCasas = s.getNCasas();
            for (int j = 0; j < s.getAsociados().size(); j++)
                if (s.getAsociados().get(j).getNCasas() < minNumCasas)
                    minNumCasas = s.getAsociados().get(j).getNCasas();
            if ((s.getNCasas() == maxNumCasas) && (s.getNCasas() > 0))
            {
                solaresConstruidos.add(s);
            }
        }        
        return solaresConstruidos;
    }
    
    public List<Casilla> getPropiedadesVendibles() {
        
       ArrayList<Casilla> propiedadesVendibles = new ArrayList<Casilla>();
                       
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < solar.size(); i++)
        {
            Solar s = solar.get(i);
            // comprobar si tambin posee los asociados
            boolean estanTodos = true;
            for (int j = 0; j < s.getAsociados().size(); j++)
            {
//                if (!solar.contains(s.getAsociados().get(j)))
//                    estanTodos = false;
                 if (!tieneSolar(s.getAsociados().get(j).getDescripcion()))
                    estanTodos = false;
            }
            if (estanTodos)
            {
                // vemos si algun solar esta edificado
                boolean algunoTieneCasa = false;

                if ((s.getNCasas() > 0) )
                        algunoTieneCasa = true;

                for (int j = 0; j < s.getAsociados().size(); j++)
                    if (s.getAsociados().get(j).getNCasas() > 0)
                            algunoTieneCasa = true;

                if (!algunoTieneCasa)
                {
                    propiedadesVendibles.add(s);
                }

            }
            else
                propiedadesVendibles.add(s);
                
        }
       for (int i =0; i<servicio.size();i++)
           propiedadesVendibles.add(servicio.get(i));
       for (int i =0;i<estacion.size();i++)
           propiedadesVendibles.add(estacion.get(i));
               
        return propiedadesVendibles;
    }
     
    public List<Casilla> getPropiedadesHipotecables() {

    List<Casilla> propiedadesHipotecables = new ArrayList<Casilla>();
    List<Casilla> todasPropiedades = new ArrayList<Casilla>();

    todasPropiedades = this.getTodasPropiedades();


    // recorremos todas las propiedades del jugador
    for (int i = 0; i < todasPropiedades.size(); i++)
    {
        if (todasPropiedades.get(i) instanceof Solar){
            if (!(todasPropiedades.get(i).isHipotecada()))
            {
                Solar s = (Solar) todasPropiedades.get(i);
                if (s.getNCasas() == 0)
                    propiedadesHipotecables.add(todasPropiedades.get(i));       
            }
        }else{
            if (!(todasPropiedades.get(i).isHipotecada())){
                propiedadesHipotecables.add(todasPropiedades.get(i));       
            }
        }           
    }
    return propiedadesHipotecables;
}

    public List<Casilla> getPropiedadesHipotecadas() {
        
        List<Casilla> propiedadesHipotecadas = new ArrayList<Casilla>();
        List<Casilla> todasPropiedades = new ArrayList<Casilla>();
        
        todasPropiedades = this.getTodasPropiedades();
        
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < todasPropiedades.size(); i++)
        {
            if ((todasPropiedades.get(i) instanceof Solar) || (todasPropiedades.get(i) instanceof Estacion) || (todasPropiedades.get(i) instanceof Servicio)){
                if (todasPropiedades.get(i).isHipotecada())
                propiedadesHipotecadas.add(todasPropiedades.get(i));
            } 
        }
        return propiedadesHipotecadas;
    }
    
    public List<Solar> getMonopolioSolares() {
        
       ArrayList<Solar> monopolio = new ArrayList<Solar>();
       Solar s;
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < solar.size(); i++)
        {
            s=solar.get(i);
            // comprobar si tambin posee los asociados
            boolean estanTodos = true;
            
            for (int j = 0; j < s.getAsociados().size(); j++)
            {
//                if (!solar.contains(s.getAsociados().get(j)))
//                    estanTodos = false;
                if (!tieneSolar(s.getAsociados().get(j).getDescripcion()))
                    estanTodos = false;
            } 
            if (estanTodos){   
                    monopolio.add(s);
                }
            }          
        return monopolio;
    }
    
    public List<Solar> getSolaresEdificables() {
        
        ArrayList<Solar> solaresEdificables = new ArrayList<Solar>();
        Solar s;
        // recorremos todas las propiedades del jugador
        for (int i = 0; i < solar.size(); i++)
        {
            s=solar.get(i);
            // comprobar si tambin posee los asociados
            boolean estanTodos = true;
            if ((s.isHipotecada())) estanTodos = false;
            for (int j = 0; j < s.getAsociados().size(); j++)
            {
                 if (!tieneSolar(s.getAsociados().get(j).getDescripcion()) || (s.getAsociados().get(j).isHipotecada())){
                    estanTodos = false;
                }
            } 
            if (estanTodos){   
                // obtenemos el n maximo y minimo de casas
                int maxNumCasas = 0;
                int minNumCasas = 0;
                if (s.getNCasas() > maxNumCasas)
                        maxNumCasas = s.getNCasas();
                for (int j = 0; j < s.getAsociados().size(); j++)
                    if (s.getAsociados().get(j).getNCasas() > maxNumCasas)
                        maxNumCasas = s.getAsociados().get(j).getNCasas();
                minNumCasas = maxNumCasas;
                if (s.getNCasas() < minNumCasas)
                        minNumCasas = s.getNCasas();
                for (int j = 0; j < s.getAsociados().size(); j++)
                    if (s.getAsociados().get(j).getNCasas() < minNumCasas)
                        minNumCasas = s.getAsociados().get(j).getNCasas();

                // si son iguales los maximos y minimos
                if ((minNumCasas == maxNumCasas) && (s.getNCasas()!=5) && (!s.isHipotecada()))
                {
                    solaresEdificables.add(s);
                }
                // si es menor el minimo
                if ((minNumCasas < maxNumCasas) && (s.getNCasas() == minNumCasas) && (s.getNCasas()!=5) && (!s.isHipotecada()))
                {
                    solaresEdificables.add(s);
                }
            }  
        }        
        return solaresEdificables;
    }

    public int getDobles() {
        return dobles;
    }

    public void setDobles(int dobles) {
        this.dobles = dobles;
        setChanged();
        notifyObservers();
    }
    
    public void incDobles() {
        this.dobles++;
        setChanged();
        notifyObservers();
    }

        public int getCogeCarta() {
        return cogeCarta;
    }

    public void setCogeCarta(int cogeCarta) {
        this.cogeCarta = cogeCarta;
    }

    public List<Solar> getSolares() {
        return solar;
    }
    
    public void AddCartasalidacarcel(CartaSalidaCarcel c) {
        c.setPropietario(this);
        cartasalidacarcel.add(c);
        setChanged();
        notifyObservers();
    }
    
    public void ComprarCartasalidacarcel(CartaSalidaCarcel c, float precio) {
        c.setPropietario(this);
        cartasalidacarcel.add(c);
        reintegro(precio);
    }
    
    public void UsarCartasalidacarcel() {
        cartasalidacarcel.get(0).setPropietario(null);
        cartasalidacarcel.remove(0);
        this.setEncarcelado(0);
    }

    public Vector<CartaSalidaCarcel> getCartasalidacarcel() {
        return (Vector<CartaSalidaCarcel>) cartasalidacarcel;
    }

     public float bienesMetalico() {
                
        float sumaBienes = 0;
        
        //Suma Solares
        for (int i = 0; i < solar.size(); i++)
        {
            Solar propiedad = solar.get(i);
            if (!propiedad.isHipotecada()){            
                    if (propiedad.getNCasas()==5)
                        sumaBienes += propiedad.getPrecioHotel() / 2;
                    else
                        sumaBienes += (propiedad.getNCasas() * (propiedad).getPrecioCasa()) / 2;
                sumaBienes += propiedad.getPrecioCompra();
            }
        }
        //Suma Estaciones
        for (int i=0;i<estacion.size();i++){
            Estacion propiedad = estacion.get(i);
            if (!propiedad.isHipotecada()){    
                sumaBienes += propiedad.getPrecioCompra();
            }
        }
        //Suma Servicios
        for (int i=0;i<servicio.size();i++){
            Servicio propiedad = servicio.get(i);
            if (!propiedad.isHipotecada()){    
                sumaBienes += propiedad.getPrecioCompra();
            }
        }
        //Suma Saldo
        sumaBienes += saldo;
        return sumaBienes;
    }
    
    public Long getId() {
        return this.id;
    }

    public void setId(Long id) {
        this.id = id;
    }
    
    @Override
    public int hashCode() {
        int hash = 0;
        hash += (this.id != null ? this.id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Jugador)) {
            return false;
        }
        Jugador other = (Jugador)object;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) return false;
        return true;
    }

    @Override
    public String toString() {
        String res =  nombre ;
        return res;
    }
}
